@@ -1,17 +1,18 @@
 from inspect import getmembers, isfunction
 from typing import Optional
-from mac_cleanup.utils import _function
+from typing import Dict  # Generics are fun
+from .utils import function
 
 
 def load_default(
-) -> dict[str, _function]:
+) -> Dict[str, function]:
     """
     Loads the default modules
 
     Returns:
         Dict w/ the module name and the module function respectively
     """
-    import mac_cleanup.default_modules as default_modules
+    from . import default_modules
 
     # getmembers returns sorted set
     return dict(getmembers(default_modules, isfunction))
@@ -19,7 +20,7 @@ def load_default(
 
 def load_custom(
         custom_path: Optional[str],
-) -> dict[str, _function]:
+) -> Dict[str, function]:
     """
     Loads the custom modules from the given path
 
@@ -34,7 +35,7 @@ def load_custom(
     from importlib.machinery import SourceFileLoader
     from pathlib import Path
 
-    custom_modules = dict()
+    custom_modules: Dict[str, function] = dict()
     # Imports all modules from the given path
     for module in Path(custom_path).expanduser().rglob("*.py"):
         # Duplicates will be overwritten
